#include <string.h>
#include <dirsrv/slapi-plugin.h>

#define FF_OID "1.3.6.1.4.1.636.2.11.11.1"

Slapi_PluginDesc	ffext_desc = {
    "ffext",				/* plug-in identifier */
    "Georgia Institute of Technology",	/* vendor name */
    "1.0",				/* plug-in revision number */
    "forwarded-for extended op"		/* plug-in description */
};

int slapi_log_access (int level, char *fmt, ...);

int
ffext_proc (Slapi_PBlock *pb)
{
    char		*oid;
    struct berval	*bval;
    PRUint64		connid = 0;
    int			opid = 0;

    if (slapi_pblock_get (pb, SLAPI_EXT_OP_REQ_OID, &oid) != 0) {
	slapi_log_error (SLAPI_LOG_PLUGIN, "ffext_proc", "Could not get OID from request.\n");
	slapi_send_ldap_result (pb, LDAP_OPERATIONS_ERROR, NULL, "Could not get OID from request.", 0, NULL);
	return (SLAPI_PLUGIN_EXTENDED_SENT_RESULT);
    }

    if (slapi_pblock_get (pb, SLAPI_EXT_OP_REQ_VALUE, &bval) != 0) {
	slapi_log_error (SLAPI_LOG_PLUGIN, "ffext_proc", "Could not get value from request.\n");
	slapi_send_ldap_result (pb, LDAP_OPERATIONS_ERROR, NULL, "Could not get value from request.", 0, NULL);
	return (SLAPI_PLUGIN_EXTENDED_SENT_RESULT);
    }

    if (slapi_pblock_get (pb, SLAPI_CONN_ID, &connid) != 0) {
	slapi_log_error (SLAPI_LOG_PLUGIN, "ffext_proc", "Could not get connection id from request.\n");
	slapi_send_ldap_result (pb, LDAP_OPERATIONS_ERROR, NULL, "Could not get connection id from request.", 0, NULL);
	return (SLAPI_PLUGIN_EXTENDED_SENT_RESULT);
    }

    if (slapi_pblock_get (pb, SLAPI_OPERATION_ID, &opid) != 0) {
	slapi_log_error (SLAPI_LOG_PLUGIN, "ffext_proc", "Could not get operation id from request.\n");
	slapi_send_ldap_result (pb, LDAP_OPERATIONS_ERROR, NULL, "Could not get operation id from request.", 0, NULL);
	return (SLAPI_PLUGIN_EXTENDED_SENT_RESULT);
    }

    slapi_log_access (256, "conn=%" NSPRIu64 " op=%d forwarded for %s\n", connid, opid, bval->bv_val);
    return (0);
}

int
ffext_init (Slapi_PBlock *pb)
{
    char	**argv, *oid, **oidlist, **namelist;
    int		rc;

    if (slapi_pblock_get (pb, SLAPI_PLUGIN_ARGV, &argv) != 0) {
	slapi_log_error (SLAPI_LOG_FATAL, "ffext_init", "Could not get argv\n");
	return (-1);
    }
    if (argv == NULL || strcmp (argv[0], FF_OID) != 0) {
	slapi_log_error (SLAPI_LOG_FATAL, "ffext_init", "OID is missing or is not %s\n", FF_OID);
	return (-1);
    }
    else {
	oid = slapi_ch_strdup (argv[0]);
	slapi_log_error (SLAPI_LOG_PLUGIN, "ffext_init", "Registering plug-in for extended op %s.\n", oid);
    }
    oidlist = (char **) slapi_ch_malloc (2 * sizeof (char *));
    oidlist[0] = oid;
    oidlist[1] = NULL;
    namelist = (char **) slapi_ch_malloc (2 * sizeof (char *));
    namelist[0] = "forwarded-for extended op";
    namelist[1] = NULL;

    rc = 0;
    rc |= slapi_pblock_set (pb, SLAPI_PLUGIN_VERSION, SLAPI_PLUGIN_CURRENT_VERSION);
    rc |= slapi_pblock_set (pb, SLAPI_PLUGIN_DESCRIPTION, (void *) &ffext_desc);
    rc |= slapi_pblock_set (pb, SLAPI_PLUGIN_EXT_OP_FN, (void *) ffext_proc);
    rc |= slapi_pblock_set (pb, SLAPI_PLUGIN_EXT_OP_OIDLIST, oidlist);
    rc |= slapi_pblock_set (pb, SLAPI_PLUGIN_EXT_OP_NAMELIST, namelist);
    if (rc) {
	slapi_log_error (SLAPI_LOG_FATAL, "ffext_init", "Failed to set plug-in details.\n");
	return (-1);
    }
    return (0);
}
